/*
** main.c
*/
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include <stdlib.h>
#include "lcd.h"
#include "utils.h"

void verify_int_fst_left() {
	if ((PINA & (1 << PA0)) == 0) {
		player2_y++;
		if (player2_y > MIDDLE_WALL)
			player2_y = MIDDLE_WALL;
		_delay_ms(PAUSE_65_MSEC);
	}
}

void verify_int_fst_right() {
	if ((PINA & (1 << PA1)) == 0) {
		player2_y--;
		if (player2_y < LEFT_WALL)
			player2_y = LEFT_WALL;
		_delay_ms(PAUSE_65_MSEC);
	}
}

void verify_int_scd_left() {
	if ((PINA & (1 << PA2)) == 0)  {
		player1_y++;
		if (player1_y > MIDDLE_WALL)
			player1_y = MIDDLE_WALL;
		_delay_ms(PAUSE_65_MSEC);
	}
}

void verify_int_scd_right() {
	if ((PINA & (1 << PA3)) == 0) {
		player1_y--;
		if (player1_y < LEFT_WALL)
			player1_y = LEFT_WALL;
		_delay_ms(PAUSE_65_MSEC);
	}
}
/* rutina de tratare a intreruperilor */
ISR(PCINT0_vect)
{
	verify_int_fst_left();
	verify_int_fst_right();
	verify_int_scd_left();
	verify_int_scd_right();
}

void where_to_go() {
	ball_x += direction_x;
	ball_y += direction_y;
}

void update_y_ball() {
	if (ball_y >= MIDDLE_WALL)
		direction_y = DIRECTION_DOWN;
	if (ball_y <= LEFT_WALL)
		direction_y = DIRECTION_UP;
}

void display_data() {
	_delay_ms(PAUSE_400_MSEC);
	lcd_clear();
	/* desenam platformele */
	lcd_goto_xy(LEFT_WALL, player1_y);
	lcd_str("I");
	lcd_goto_xy(RIGHT_WALL, player2_y);
	lcd_str("I");

	/* desenam scorul */
	lcd_goto_xy(3, 1);
	itoa(score_1, buffer, 10);
	lcd_str(buffer);
	lcd_goto_xy(12, 1);
	itoa(score_2, buffer, 10);
	lcd_str(buffer);

	/* desenam mingea */
	lcd_goto_xy(ball_x, ball_y);
	lcd_chr('o');
}

void pong_play() {
	for (;;) {
		where_to_go();
		update_y_ball();
		
		if (ball_x >= RIGHT_WALL) {
			if (ball_y == player2_y || ball_y == player2_y - 1 || ball_y == player2_y + 1) {
				direction_x *= (-1);
			} else {
				/* resetam pozitia bilei */
				ball_x = BALL_X_DEFAULT;
				ball_y = BALL_Y_DEFAULT;
				score_1++;
				if (score_1 == MAX_SCORE){
					/* castigatorul este primul jucator */
					lcd_clear();
					lcd_goto_xy(2, 2);
					lcd_str("Player 1 won!");
					break;
				}
			}
		} else if (ball_x <= LEFT_WALL){
			if (ball_y == player1_y || ball_y == player1_y - 1 || ball_y == player1_y + 1) {
				direction_x *= (-1);
			} else {
				/* resetam pozitia bilei */
				ball_x = BALL_X_DEFAULT;
				ball_y = BALL_Y_DEFAULT;
				score_2++;
				if (score_2 == MAX_SCORE) {
					/* castigatorul este al doilea jucator */
					lcd_clear();
					lcd_goto_xy(2, 2);
					lcd_str("Player 2 won!");
					break;
				}
			}
		}
		display_data();		
	}
}

int main(void)
{
	/* initiaizare ecran */
	lcd_init();

	/* initializam butoanele cu care vom controla plaformele
	PAO/PA1 - directia stanga/dreapta jucator1
	PA2/PA3 - directia stanga/dreapta jucator2*/
	
	DDRA &= ~(1 << PA0);
	PORTA |= (1 << PA0);

	DDRA &= ~(1 << PA1);
	PORTA |= (1 << PA1);

	DDRA &= ~(1 << PA2);
	PORTA |= (1 << PA2);

	DDRA &= ~(1 << PA3);
	PORTA |= (1 << PA3);

	/* setam contrastul */
	lcd_contrast(0x40);

	/* scriem un mesaj care apare la inceput pe ecran */
	lcd_goto_xy(2, 2);
	lcd_str("Pong PM 2019");
	lcd_goto_xy(2, 4);
	lcd_str("First to 5");
	_delay_ms(PAUSE_2_SEC);
	lcd_clear();

	/* setam intreruperile pentru butoane*/
	sei();
	PCICR |= (1 << PCIE0);
    PCMSK0 |= (1 << PCINT0);
    PCMSK0 |= (1 << PCINT1);
    PCMSK0 |= (1 << PCINT2);
    PCMSK0 |= (1 << PCINT3);
	
	/* acum trebuie doar sa jucam */
	pong_play();
	return 0;
}
